from codecs import getdecoder


class LangWhat:
    def __init__(
        self,
        query,
        is_zh: bool = False,
        sydney: bool = False,
        bing_cookie_json_path: str | None = None,
        api_base: str | None = None,
    ):
        # if api_base:
        #     openai.api_base = api_base
        self.query = query
        self.is_zh = is_zh
        self.sydney = sydney
        if self.sydney and bing_cookie_json_path is None:
            raise Exception("Bing cookie json path not provided")
        self.cookie_path = bing_cookie_json_path
        self.references: str | None = None

    def get_response_openai(self):
        from langwhat.utils import get_llm_chain

        chain = get_llm_chain(is_zh=self.is_zh)
        response = chain(self.query)
        from .utils import parse_chain_response

        self.might_be, self.description = parse_chain_response(response)

    async def get_response_sydney(self):
        from langwhat.utils import get_edgegpt_answer, split_edgegpt_answer

        assert self.cookie_path is not None
        edgegpt_answer = await get_edgegpt_answer(
            cookie_path=self.cookie_path, query=self.query, is_zh=self.is_zh
        )
        references, self.might_be, self.description = split_edgegpt_answer(
            edgegpt_answer
        )
        if references:
            self.references = references

    async def show(self):
        from rich.console import Console
        from rich.table import Table
        from rich.style import Style
        from rich.text import Text

        if self.sydney:
            await self.get_response_sydney()
        else:
            self.get_response_openai()

        console = Console()
        title = Text(
            "LangWhat [https://github.com/tddschn/langwhat]",
            style=Style(color="#268bd2", bold=True),
        )
        table = Table(title=title, show_lines=False, style="dim")  # type: ignore
        table.add_column("Query", style=Style(color="#b58900"))
        table.add_column("Might Be", style=Style(color="#d33682"), justify="middle")  # type: ignore
        table.add_column("Description", style=Style(color="#859900"), justify="left")
        table.add_row(self.query, self.might_be, self.description)
        console.print(table)

        if self.references:
            console.print("References:")
            console.print(self.references)



from typing import TYPE_CHECKING


if TYPE_CHECKING:
    from langchain.chains import LLMChain
    from langchain.prompts.few_shot import FewShotPromptTemplate
    from EdgeGPT import Chatbot


def get_prompt(is_zh: bool = False, sydney: bool = False) -> 'FewShotPromptTemplate':
    from langchain.prompts.few_shot import FewShotPromptTemplate
    from langchain.prompts.prompt import PromptTemplate

    langwhat_example_prompt = PromptTemplate(
        input_variables=["question", "answer"], template="Q: {question}\n{answer}"
    )

    langwhat_examples = [
        {
            "question": "vJzDRsEKDa0",
            "answer": """
Might be:
A YouTube video ID
Description:
The ID "vJzDRsEKDa0" is most likely a unique identifier for a specific video on YouTube.
This alphanumeric code is automatically assigned to every video uploaded on the platform and is used to access the video directly or share it with other users.
""",
        },
        {
            "question": "https://langchain.readthedocs.io/",
            "answer": """
Might be:
A website or online documentation
Description:
This website/document provides information about LangChain, a technology or platform that could be used for language processing or analysis.
    """,
        },
    ]

    langwhat_examples_zh = [
        {
            "question": "vJzDRsEKDa0",
            "answer": """
可能是:
YouTube 视频 ID
描述:
“vJzDRsEKDa0”这个ID很可能是YouTube上特定视频的唯一标识符。
每上传一个视频，该平台都会自动分配这个字母数字代码，并用于直接访问或与其他用户共享该视频。
""",
        },
        {
            "question": "https://langchain.readthedocs.io/",
            "answer": """
可能是:
一个网站或在线文档
描述:
这个网站/文件提供有关LangChain的信息，它是一种可用于语言处理或分析的技术或平台。
    """,
        },
    ]

    prefix_sydney = '''
You are a helpful assistant that answer questions in the format below,
you must answer the question only, do not ask what I want to know more about:
"""
'''
    suffix_sydney = '''"""\n\nBEGIN!\n\nQ: {input}'''
    suffix_openai = '''\nQ: {input}'''
    langwhat_prompt = FewShotPromptTemplate(
        example_prompt=langwhat_example_prompt,
        examples=langwhat_examples_zh if is_zh else langwhat_examples,
        suffix=suffix_sydney if sydney else suffix_openai,
        prefix=prefix_sydney if sydney else '',
        input_variables=["input"],
    )
    return langwhat_prompt


def get_llm_chain(is_zh: bool = False) -> 'LLMChain':
    from langchain.llms import OpenAIChat
    from langchain.chains import LLMChain

    llm = OpenAIChat()  # type: ignore
    langwhat_prompt = get_prompt(is_zh=is_zh)
    chain = LLMChain(llm=llm, prompt=langwhat_prompt)
    return chain


async def get_edgegpt_answer(
    cookie_path: str,
    query: str,
    is_zh: bool = False,
) -> str:
    from EdgeGPT import Chatbot

    bot = Chatbot(
        cookiePath=cookie_path,
    )
    langwhat_prompt = get_prompt(is_zh=is_zh, sydney=True)
    prompt = langwhat_prompt.format(input=query)
    response = await bot.ask(prompt)
    answer = response["item"]["messages"][1]["adaptiveCards"][0]["body"][0]["text"]
    return answer


def parse_standard_answer_format(answer: str) -> tuple[str, str]:
    ans_lines = answer.strip().splitlines()
    might_be = ans_lines[1]
    description = '\n'.join(ans_lines[3:])
    description = description.replace("\n", ",\r\n")
    # description = description.replace(",", ",\r\n")
    # description = description.replace("，", "，\r\n")
    # description = description.replace("。", "。\r\n")
    # description = description.replace(".", ".\r\n")
    return might_be, description


def parse_chain_response(chain_response: dict[str, str]) -> tuple[str, str]:
    """Parse the response from the chain.

    Args:
        chain_response (dict[str, str]): The response from the chain.

    Returns:
        tuple[str, str]: The first element is the might be, the second element is the description.
    """
    answer = chain_response['text']
    return parse_standard_answer_format(answer)


def split_edgegpt_answer(answer: str) -> tuple[str, str, str]:
    answer = answer.strip()
    if answer.startswith('['):
        references, answer_body = answer.split('\n\n', maxsplit=1)
        references = references.strip()
        answer_body = answer_body.strip()
    else:
        references = ''
        answer_body = answer
    return references, *parse_standard_answer_format(answer_body)


# def save_prompts(
#     langwhat_prompt: 'FewShotPromptTemplate',
#     langwhat_prompt_zh: 'FewShotPromptTemplate',
# ):
#     from pathlib import Path

#     base_dir = Path(__file__).parent
#     for ext in ('.yaml', '.json'):
#         for i, prompt in enumerate((langwhat_prompt, langwhat_prompt_zh)):
#             if i == 0:
#                 prompt.save(base_dir / f'langwhat_prompt{ext}')
#             else:
#                 prompt.save(base_dir / f'langwhat_prompt_zh{ext}')



from pathlib import Path

XDG_CONFIG_DIR = Path.home() / '.config'
OPENAI_API_KEY_FILE = Path(XDG_CONFIG_DIR) / "langwhat" / "api_key.txt"
EDGEGPT_COOKIE_FILE = Path(XDG_CONFIG_DIR) / "langwhat" / "cookie_file_path.txt"


def get_api_key() -> str:
    """
    Retrieves API key from the file located in the user's home
    directory, or prompts the user to input it if it does not exist.

    :return: String API key for OpenAI API requests.
    """
    if not OPENAI_API_KEY_FILE.exists():
        print('Instructions to get your API key:')
        print('https://platform.openai.com/account/api-keys')
        from getpass import getpass

        api_key = getpass(prompt="Please enter your API secret key: ")
        OPENAI_API_KEY_FILE.parent.mkdir(parents=True, exist_ok=True)
        OPENAI_API_KEY_FILE.write_text(api_key)
    else:
        api_key = OPENAI_API_KEY_FILE.read_text().strip()
    return api_key


def get_cookies_file() -> str:
    """
    Retrieves cookie file from the file located in the user's home
    directory, or prompts the user to input it if it does not exist.

    :return: String cookie file for Bing API requests.
    """
    if not EDGEGPT_COOKIE_FILE.exists():
        print('Instructions to get your cookie file:')
        print('https://github.com/acheong08/EdgeGPT')
        from getpass import getpass

        cookie_file = getpass(
            prompt="Please enter the path to your bing.com cookie file: "
        )
        EDGEGPT_COOKIE_FILE.parent.mkdir(parents=True, exist_ok=True)
        EDGEGPT_COOKIE_FILE.write_text(cookie_file)
    else:
        cookie_file = EDGEGPT_COOKIE_FILE.read_text().strip()
    return cookie_file



#!/usr/bin/env python3

import argparse
from os import environ
from . import __version__


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("what", help="what is it")
    # parser.add_argument(
    #     "--openai-key", dest="openai_key", type=str, default="", help="OpenAI api key"
    # )
    # parser.add_argument(
    #     '--bing-cookie-json', dest='bing_cookie_json', type=str, default='', help='Bing cookie json'
    # )
    # args to change api_base
    # parser.add_argument(
    #     "--api_base",
    #     dest="api_base",
    #     type=str,
    #     help="specify base url other than the OpenAI's official API address",
    # )
    parser.add_argument(
        "-z",
        "--zh",
        dest="zh",
        action="store_true",
        help="Use Mandarin to prompt and answer",
    )

    parser.add_argument(
        '-s',
        '--sydney',
        dest='sydney',
        action='store_true',
        help='Use Sydney (Bing AI) instead of OpenAI',
    )

    parser.add_argument(
        '-V',
        '--version',
        action='version',
        version=f'%(prog)s {__version__}',
    )

    args = parser.parse_args()
    return args


def main():
    args = get_args()
    from langwhat.langwhat import LangWhat

    if not args.sydney:
        # OPENAI_API_KEY = args.openai_key or env.get("OPENAI_API_KEY")
        # BING_COOKIE_JSON = args.bing_cookie_json or env.get("BING_COOKIE_JSON") or env.get("COOKIE_FILE")
        OPENAI_API_KEY = environ.get("OPENAI_API_KEY")
        if not OPENAI_API_KEY:
            from .config import get_api_key

            OPENAI_API_KEY = get_api_key()
            environ['OPENAI_API_KEY'] = OPENAI_API_KEY
            if not OPENAI_API_KEY:
                raise Exception(
                    "OpenAI API key not provided, please provide it by running `export OPENAI_API_KEY=<sk-XXXX>`"
                )
        BING_COOKIE_JSON = None
    else:
        from .config import get_cookies_file

        BING_COOKIE_JSON = get_cookies_file()

    # what = What(args.what, is_en=args.en, api_base=args.api_base)
    langwhat = LangWhat(
        args.what,
        is_zh=args.zh,
        bing_cookie_json_path=BING_COOKIE_JSON,
        sydney=args.sydney,
        api_base=None,
    )
    import asyncio

    asyncio.run(langwhat.show())


if __name__ == "__main__":
    main()



edgegpt_answer_mary = """
[1]: https://en.wikipedia.org/wiki/Mary_Ball_Washington "Mary Ball Washington - Wikipedia"
[2]: https://www.mountvernon.org/library/digitalhistory/digital-encyclopedia/article/mary-ball-washington/ "Mary Ball Washington · George Washington's Mount Vernon"
[3]: https://washingtonindependent.com/mary-ball-washington/ "Mary Ball Washington: Who Was George Washington's Mother?"
[4]: https://www.geni.com/people/Mary-Washington/6000000007409105599 "Mary Washington (Ball) (1708 - 1789) - Genealogy - geni family tree"

Might be:
The mother of George Washington
Description:
Mary Ball Washington was born in 1708 in Virginia. She married Augustine Washington in 1730 and had six children, including George Washington, who became the first president of the United States[^1^][1] [^2^][2]. She died in 1789 at age 80[^1^][1].

What would you like to know more about Mary Ball Washington?
"""

edgegpt_answer_OPENAI_API_KEY_FILE = '''
Might be:
An environment variable or a file name
Description:
The OPENAI_API_KEY_FILE could be either an environment variable that stores the path to a file containing the OpenAI API key, or the name of the file itself. The OpenAI API key is a secret token that allows access to the OpenAI services and products.
'''
edgegpt_test_query_with_answer = {
    'mary ball washington', edgegpt_answer_mary,
    'OPENAI_API_KEY_FILE', edgegpt_answer_OPENAI_API_KEY_FILE,
}
